% Multiplication matrix / eigensystem version

% 80x56 extended multiresultant matrix for 4 point DLT-like pose method.
% Using the DLT on 4 known 3D points, the possible projection matrices
% have been reduced to a 4 parameter family P = u1*P1+..+u4*P4 where
% (u1..u4) are unknown parameters.  We use 4 supplementary calibration
% constraints aspect_ratio = 1, skew=0, principal_point=(0,0) (i.e.
% w(1,1)-w(2,2) = w(1,2) = w(1,3) = w(2,3) = 0 where w is the DIAC) to
% eliminate (u1..u4) and solve for P, camera pose, calibration, etc.
% The redundancy is 1 (since u1..u4 are only defined up to scale), so a
% linear solution is possible.  The constraints are all quadratic in
% (u1..u4). Each row of the 4x10 input matrix M represents one of these
% homogeneous quadratics (i.e. it is a flattened 4x4 symmetric
% matrix). The output multiresultant matrix has rank 79 in the noiseless
% case (but alas only 76 for coplanar points). The last 4 components of
% its null vector are (...u1..u4), which in turn gives P, pose, etc. The
% matrix was calculated with Maple using my extended multiresultant
% routine xmultires().

function [u,condit] = dlt4multires2(M);
   A = zeros(80,56);
   A(1,47) = M(1,1);
   A(2,39) = M(1,1);
   A(3,38) = M(1,1);
   A(4,37) = M(1,1);
   A(5,27) = M(1,1);
   A(6,26) = M(1,1);
   A(7,25) = M(1,1);
   A(8,24) = M(1,1);
   A(9,23) = M(1,1);
   A(10,22) = M(1,1);
   A(11,10) = M(1,1);
   A(12,9) = M(1,1);
   A(13,8) = M(1,1);
   A(14,7) = M(1,1);
   A(15,6) = M(1,1);
   A(16,5) = M(1,1);
   A(17,4) = M(1,1);
   A(18,3) = M(1,1);
   A(19,2) = M(1,1);
   A(20,1) = M(1,1);
   A(1,48) = M(1,2);
   A(2,41) = M(1,2);
   A(3,40) = M(1,2);
   A(4,38) = M(1,2);
   A(5,30) = M(1,2);
   A(6,29) = M(1,2);
   A(7,28) = M(1,2);
   A(8,26) = M(1,2);
   A(9,25) = M(1,2);
   A(10,23) = M(1,2);
   A(11,14) = M(1,2);
   A(12,13) = M(1,2);
   A(13,12) = M(1,2);
   A(14,11) = M(1,2);
   A(15,9) = M(1,2);
   A(16,8) = M(1,2);
   A(17,7) = M(1,2);
   A(18,5) = M(1,2);
   A(19,4) = M(1,2);
   A(20,2) = M(1,2);
   A(1,50) = M(1,3);
   A(2,44) = M(1,3);
   A(3,43) = M(1,3);
   A(4,40) = M(1,3);
   A(5,34) = M(1,3);
   A(6,33) = M(1,3);
   A(7,32) = M(1,3);
   A(8,29) = M(1,3);
   A(9,28) = M(1,3);
   A(10,25) = M(1,3);
   A(11,19) = M(1,3);
   A(12,18) = M(1,3);
   A(13,17) = M(1,3);
   A(14,16) = M(1,3);
   A(15,13) = M(1,3);
   A(16,12) = M(1,3);
   A(17,11) = M(1,3);
   A(18,8) = M(1,3);
   A(19,7) = M(1,3);
   A(20,4) = M(1,3);
   A(1,49) = M(1,4);
   A(2,42) = M(1,4);
   A(3,41) = M(1,4);
   A(4,39) = M(1,4);
   A(5,31) = M(1,4);
   A(6,30) = M(1,4);
   A(7,29) = M(1,4);
   A(8,27) = M(1,4);
   A(9,26) = M(1,4);
   A(10,24) = M(1,4);
   A(11,15) = M(1,4);
   A(12,14) = M(1,4);
   A(13,13) = M(1,4);
   A(14,12) = M(1,4);
   A(15,10) = M(1,4);
   A(16,9) = M(1,4);
   A(17,8) = M(1,4);
   A(18,6) = M(1,4);
   A(19,5) = M(1,4);
   A(20,3) = M(1,4);
   A(1,51) = M(1,5);
   A(2,45) = M(1,5);
   A(3,44) = M(1,5);
   A(4,41) = M(1,5);
   A(5,35) = M(1,5);
   A(6,34) = M(1,5);
   A(7,33) = M(1,5);
   A(8,30) = M(1,5);
   A(9,29) = M(1,5);
   A(10,26) = M(1,5);
   A(11,20) = M(1,5);
   A(12,19) = M(1,5);
   A(13,18) = M(1,5);
   A(14,17) = M(1,5);
   A(15,14) = M(1,5);
   A(16,13) = M(1,5);
   A(17,12) = M(1,5);
   A(18,9) = M(1,5);
   A(19,8) = M(1,5);
   A(20,5) = M(1,5);
   A(1,52) = M(1,6);
   A(2,46) = M(1,6);
   A(3,45) = M(1,6);
   A(4,42) = M(1,6);
   A(5,36) = M(1,6);
   A(6,35) = M(1,6);
   A(7,34) = M(1,6);
   A(8,31) = M(1,6);
   A(9,30) = M(1,6);
   A(10,27) = M(1,6);
   A(11,21) = M(1,6);
   A(12,20) = M(1,6);
   A(13,19) = M(1,6);
   A(14,18) = M(1,6);
   A(15,15) = M(1,6);
   A(16,14) = M(1,6);
   A(17,13) = M(1,6);
   A(18,10) = M(1,6);
   A(19,9) = M(1,6);
   A(20,6) = M(1,6);
   A(1,53) = M(1,7);
   A(2,49) = M(1,7);
   A(3,48) = M(1,7);
   A(4,47) = M(1,7);
   A(5,42) = M(1,7);
   A(6,41) = M(1,7);
   A(7,40) = M(1,7);
   A(8,39) = M(1,7);
   A(9,38) = M(1,7);
   A(10,37) = M(1,7);
   A(11,31) = M(1,7);
   A(12,30) = M(1,7);
   A(13,29) = M(1,7);
   A(14,28) = M(1,7);
   A(15,27) = M(1,7);
   A(16,26) = M(1,7);
   A(17,25) = M(1,7);
   A(18,24) = M(1,7);
   A(19,23) = M(1,7);
   A(20,22) = M(1,7);
   A(1,54) = M(1,8);
   A(2,51) = M(1,8);
   A(3,50) = M(1,8);
   A(4,48) = M(1,8);
   A(5,45) = M(1,8);
   A(6,44) = M(1,8);
   A(7,43) = M(1,8);
   A(8,41) = M(1,8);
   A(9,40) = M(1,8);
   A(10,38) = M(1,8);
   A(11,35) = M(1,8);
   A(12,34) = M(1,8);
   A(13,33) = M(1,8);
   A(14,32) = M(1,8);
   A(15,30) = M(1,8);
   A(16,29) = M(1,8);
   A(17,28) = M(1,8);
   A(18,26) = M(1,8);
   A(19,25) = M(1,8);
   A(20,23) = M(1,8);
   A(1,55) = M(1,9);
   A(2,52) = M(1,9);
   A(3,51) = M(1,9);
   A(4,49) = M(1,9);
   A(5,46) = M(1,9);
   A(6,45) = M(1,9);
   A(7,44) = M(1,9);
   A(8,42) = M(1,9);
   A(9,41) = M(1,9);
   A(10,39) = M(1,9);
   A(11,36) = M(1,9);
   A(12,35) = M(1,9);
   A(13,34) = M(1,9);
   A(14,33) = M(1,9);
   A(15,31) = M(1,9);
   A(16,30) = M(1,9);
   A(17,29) = M(1,9);
   A(18,27) = M(1,9);
   A(19,26) = M(1,9);
   A(20,24) = M(1,9);
   A(1,56) = M(1,10);
   A(2,55) = M(1,10);
   A(3,54) = M(1,10);
   A(4,53) = M(1,10);
   A(5,52) = M(1,10);
   A(6,51) = M(1,10);
   A(7,50) = M(1,10);
   A(8,49) = M(1,10);
   A(9,48) = M(1,10);
   A(10,47) = M(1,10);
   A(11,46) = M(1,10);
   A(12,45) = M(1,10);
   A(13,44) = M(1,10);
   A(14,43) = M(1,10);
   A(15,42) = M(1,10);
   A(16,41) = M(1,10);
   A(17,40) = M(1,10);
   A(18,39) = M(1,10);
   A(19,38) = M(1,10);
   A(20,37) = M(1,10);
   A(21,47) = M(2,1);
   A(22,39) = M(2,1);
   A(23,38) = M(2,1);
   A(24,37) = M(2,1);
   A(25,27) = M(2,1);
   A(26,26) = M(2,1);
   A(27,25) = M(2,1);
   A(28,24) = M(2,1);
   A(29,23) = M(2,1);
   A(30,22) = M(2,1);
   A(31,10) = M(2,1);
   A(32,9) = M(2,1);
   A(33,8) = M(2,1);
   A(34,7) = M(2,1);
   A(35,6) = M(2,1);
   A(36,5) = M(2,1);
   A(37,4) = M(2,1);
   A(38,3) = M(2,1);
   A(39,2) = M(2,1);
   A(40,1) = M(2,1);
   A(21,48) = M(2,2);
   A(22,41) = M(2,2);
   A(23,40) = M(2,2);
   A(24,38) = M(2,2);
   A(25,30) = M(2,2);
   A(26,29) = M(2,2);
   A(27,28) = M(2,2);
   A(28,26) = M(2,2);
   A(29,25) = M(2,2);
   A(30,23) = M(2,2);
   A(31,14) = M(2,2);
   A(32,13) = M(2,2);
   A(33,12) = M(2,2);
   A(34,11) = M(2,2);
   A(35,9) = M(2,2);
   A(36,8) = M(2,2);
   A(37,7) = M(2,2);
   A(38,5) = M(2,2);
   A(39,4) = M(2,2);
   A(40,2) = M(2,2);
   A(21,50) = M(2,3);
   A(22,44) = M(2,3);
   A(23,43) = M(2,3);
   A(24,40) = M(2,3);
   A(25,34) = M(2,3);
   A(26,33) = M(2,3);
   A(27,32) = M(2,3);
   A(28,29) = M(2,3);
   A(29,28) = M(2,3);
   A(30,25) = M(2,3);
   A(31,19) = M(2,3);
   A(32,18) = M(2,3);
   A(33,17) = M(2,3);
   A(34,16) = M(2,3);
   A(35,13) = M(2,3);
   A(36,12) = M(2,3);
   A(37,11) = M(2,3);
   A(38,8) = M(2,3);
   A(39,7) = M(2,3);
   A(40,4) = M(2,3);
   A(21,49) = M(2,4);
   A(22,42) = M(2,4);
   A(23,41) = M(2,4);
   A(24,39) = M(2,4);
   A(25,31) = M(2,4);
   A(26,30) = M(2,4);
   A(27,29) = M(2,4);
   A(28,27) = M(2,4);
   A(29,26) = M(2,4);
   A(30,24) = M(2,4);
   A(31,15) = M(2,4);
   A(32,14) = M(2,4);
   A(33,13) = M(2,4);
   A(34,12) = M(2,4);
   A(35,10) = M(2,4);
   A(36,9) = M(2,4);
   A(37,8) = M(2,4);
   A(38,6) = M(2,4);
   A(39,5) = M(2,4);
   A(40,3) = M(2,4);
   A(21,51) = M(2,5);
   A(22,45) = M(2,5);
   A(23,44) = M(2,5);
   A(24,41) = M(2,5);
   A(25,35) = M(2,5);
   A(26,34) = M(2,5);
   A(27,33) = M(2,5);
   A(28,30) = M(2,5);
   A(29,29) = M(2,5);
   A(30,26) = M(2,5);
   A(31,20) = M(2,5);
   A(32,19) = M(2,5);
   A(33,18) = M(2,5);
   A(34,17) = M(2,5);
   A(35,14) = M(2,5);
   A(36,13) = M(2,5);
   A(37,12) = M(2,5);
   A(38,9) = M(2,5);
   A(39,8) = M(2,5);
   A(40,5) = M(2,5);
   A(21,52) = M(2,6);
   A(22,46) = M(2,6);
   A(23,45) = M(2,6);
   A(24,42) = M(2,6);
   A(25,36) = M(2,6);
   A(26,35) = M(2,6);
   A(27,34) = M(2,6);
   A(28,31) = M(2,6);
   A(29,30) = M(2,6);
   A(30,27) = M(2,6);
   A(31,21) = M(2,6);
   A(32,20) = M(2,6);
   A(33,19) = M(2,6);
   A(34,18) = M(2,6);
   A(35,15) = M(2,6);
   A(36,14) = M(2,6);
   A(37,13) = M(2,6);
   A(38,10) = M(2,6);
   A(39,9) = M(2,6);
   A(40,6) = M(2,6);
   A(21,53) = M(2,7);
   A(22,49) = M(2,7);
   A(23,48) = M(2,7);
   A(24,47) = M(2,7);
   A(25,42) = M(2,7);
   A(26,41) = M(2,7);
   A(27,40) = M(2,7);
   A(28,39) = M(2,7);
   A(29,38) = M(2,7);
   A(30,37) = M(2,7);
   A(31,31) = M(2,7);
   A(32,30) = M(2,7);
   A(33,29) = M(2,7);
   A(34,28) = M(2,7);
   A(35,27) = M(2,7);
   A(36,26) = M(2,7);
   A(37,25) = M(2,7);
   A(38,24) = M(2,7);
   A(39,23) = M(2,7);
   A(40,22) = M(2,7);
   A(21,54) = M(2,8);
   A(22,51) = M(2,8);
   A(23,50) = M(2,8);
   A(24,48) = M(2,8);
   A(25,45) = M(2,8);
   A(26,44) = M(2,8);
   A(27,43) = M(2,8);
   A(28,41) = M(2,8);
   A(29,40) = M(2,8);
   A(30,38) = M(2,8);
   A(31,35) = M(2,8);
   A(32,34) = M(2,8);
   A(33,33) = M(2,8);
   A(34,32) = M(2,8);
   A(35,30) = M(2,8);
   A(36,29) = M(2,8);
   A(37,28) = M(2,8);
   A(38,26) = M(2,8);
   A(39,25) = M(2,8);
   A(40,23) = M(2,8);
   A(21,55) = M(2,9);
   A(22,52) = M(2,9);
   A(23,51) = M(2,9);
   A(24,49) = M(2,9);
   A(25,46) = M(2,9);
   A(26,45) = M(2,9);
   A(27,44) = M(2,9);
   A(28,42) = M(2,9);
   A(29,41) = M(2,9);
   A(30,39) = M(2,9);
   A(31,36) = M(2,9);
   A(32,35) = M(2,9);
   A(33,34) = M(2,9);
   A(34,33) = M(2,9);
   A(35,31) = M(2,9);
   A(36,30) = M(2,9);
   A(37,29) = M(2,9);
   A(38,27) = M(2,9);
   A(39,26) = M(2,9);
   A(40,24) = M(2,9);
   A(21,56) = M(2,10);
   A(22,55) = M(2,10);
   A(23,54) = M(2,10);
   A(24,53) = M(2,10);
   A(25,52) = M(2,10);
   A(26,51) = M(2,10);
   A(27,50) = M(2,10);
   A(28,49) = M(2,10);
   A(29,48) = M(2,10);
   A(30,47) = M(2,10);
   A(31,46) = M(2,10);
   A(32,45) = M(2,10);
   A(33,44) = M(2,10);
   A(34,43) = M(2,10);
   A(35,42) = M(2,10);
   A(36,41) = M(2,10);
   A(37,40) = M(2,10);
   A(38,39) = M(2,10);
   A(39,38) = M(2,10);
   A(40,37) = M(2,10);
   A(41,47) = M(3,1);
   A(42,39) = M(3,1);
   A(43,38) = M(3,1);
   A(44,37) = M(3,1);
   A(45,27) = M(3,1);
   A(46,26) = M(3,1);
   A(47,25) = M(3,1);
   A(48,24) = M(3,1);
   A(49,23) = M(3,1);
   A(50,22) = M(3,1);
   A(51,10) = M(3,1);
   A(52,9) = M(3,1);
   A(53,8) = M(3,1);
   A(54,7) = M(3,1);
   A(55,6) = M(3,1);
   A(56,5) = M(3,1);
   A(57,4) = M(3,1);
   A(58,3) = M(3,1);
   A(59,2) = M(3,1);
   A(60,1) = M(3,1);
   A(41,48) = M(3,2);
   A(42,41) = M(3,2);
   A(43,40) = M(3,2);
   A(44,38) = M(3,2);
   A(45,30) = M(3,2);
   A(46,29) = M(3,2);
   A(47,28) = M(3,2);
   A(48,26) = M(3,2);
   A(49,25) = M(3,2);
   A(50,23) = M(3,2);
   A(51,14) = M(3,2);
   A(52,13) = M(3,2);
   A(53,12) = M(3,2);
   A(54,11) = M(3,2);
   A(55,9) = M(3,2);
   A(56,8) = M(3,2);
   A(57,7) = M(3,2);
   A(58,5) = M(3,2);
   A(59,4) = M(3,2);
   A(60,2) = M(3,2);
   A(41,50) = M(3,3);
   A(42,44) = M(3,3);
   A(43,43) = M(3,3);
   A(44,40) = M(3,3);
   A(45,34) = M(3,3);
   A(46,33) = M(3,3);
   A(47,32) = M(3,3);
   A(48,29) = M(3,3);
   A(49,28) = M(3,3);
   A(50,25) = M(3,3);
   A(51,19) = M(3,3);
   A(52,18) = M(3,3);
   A(53,17) = M(3,3);
   A(54,16) = M(3,3);
   A(55,13) = M(3,3);
   A(56,12) = M(3,3);
   A(57,11) = M(3,3);
   A(58,8) = M(3,3);
   A(59,7) = M(3,3);
   A(60,4) = M(3,3);
   A(41,49) = M(3,4);
   A(42,42) = M(3,4);
   A(43,41) = M(3,4);
   A(44,39) = M(3,4);
   A(45,31) = M(3,4);
   A(46,30) = M(3,4);
   A(47,29) = M(3,4);
   A(48,27) = M(3,4);
   A(49,26) = M(3,4);
   A(50,24) = M(3,4);
   A(51,15) = M(3,4);
   A(52,14) = M(3,4);
   A(53,13) = M(3,4);
   A(54,12) = M(3,4);
   A(55,10) = M(3,4);
   A(56,9) = M(3,4);
   A(57,8) = M(3,4);
   A(58,6) = M(3,4);
   A(59,5) = M(3,4);
   A(60,3) = M(3,4);
   A(41,51) = M(3,5);
   A(42,45) = M(3,5);
   A(43,44) = M(3,5);
   A(44,41) = M(3,5);
   A(45,35) = M(3,5);
   A(46,34) = M(3,5);
   A(47,33) = M(3,5);
   A(48,30) = M(3,5);
   A(49,29) = M(3,5);
   A(50,26) = M(3,5);
   A(51,20) = M(3,5);
   A(52,19) = M(3,5);
   A(53,18) = M(3,5);
   A(54,17) = M(3,5);
   A(55,14) = M(3,5);
   A(56,13) = M(3,5);
   A(57,12) = M(3,5);
   A(58,9) = M(3,5);
   A(59,8) = M(3,5);
   A(60,5) = M(3,5);
   A(41,52) = M(3,6);
   A(42,46) = M(3,6);
   A(43,45) = M(3,6);
   A(44,42) = M(3,6);
   A(45,36) = M(3,6);
   A(46,35) = M(3,6);
   A(47,34) = M(3,6);
   A(48,31) = M(3,6);
   A(49,30) = M(3,6);
   A(50,27) = M(3,6);
   A(51,21) = M(3,6);
   A(52,20) = M(3,6);
   A(53,19) = M(3,6);
   A(54,18) = M(3,6);
   A(55,15) = M(3,6);
   A(56,14) = M(3,6);
   A(57,13) = M(3,6);
   A(58,10) = M(3,6);
   A(59,9) = M(3,6);
   A(60,6) = M(3,6);
   A(41,53) = M(3,7);
   A(42,49) = M(3,7);
   A(43,48) = M(3,7);
   A(44,47) = M(3,7);
   A(45,42) = M(3,7);
   A(46,41) = M(3,7);
   A(47,40) = M(3,7);
   A(48,39) = M(3,7);
   A(49,38) = M(3,7);
   A(50,37) = M(3,7);
   A(51,31) = M(3,7);
   A(52,30) = M(3,7);
   A(53,29) = M(3,7);
   A(54,28) = M(3,7);
   A(55,27) = M(3,7);
   A(56,26) = M(3,7);
   A(57,25) = M(3,7);
   A(58,24) = M(3,7);
   A(59,23) = M(3,7);
   A(60,22) = M(3,7);
   A(41,54) = M(3,8);
   A(42,51) = M(3,8);
   A(43,50) = M(3,8);
   A(44,48) = M(3,8);
   A(45,45) = M(3,8);
   A(46,44) = M(3,8);
   A(47,43) = M(3,8);
   A(48,41) = M(3,8);
   A(49,40) = M(3,8);
   A(50,38) = M(3,8);
   A(51,35) = M(3,8);
   A(52,34) = M(3,8);
   A(53,33) = M(3,8);
   A(54,32) = M(3,8);
   A(55,30) = M(3,8);
   A(56,29) = M(3,8);
   A(57,28) = M(3,8);
   A(58,26) = M(3,8);
   A(59,25) = M(3,8);
   A(60,23) = M(3,8);
   A(41,55) = M(3,9);
   A(42,52) = M(3,9);
   A(43,51) = M(3,9);
   A(44,49) = M(3,9);
   A(45,46) = M(3,9);
   A(46,45) = M(3,9);
   A(47,44) = M(3,9);
   A(48,42) = M(3,9);
   A(49,41) = M(3,9);
   A(50,39) = M(3,9);
   A(51,36) = M(3,9);
   A(52,35) = M(3,9);
   A(53,34) = M(3,9);
   A(54,33) = M(3,9);
   A(55,31) = M(3,9);
   A(56,30) = M(3,9);
   A(57,29) = M(3,9);
   A(58,27) = M(3,9);
   A(59,26) = M(3,9);
   A(60,24) = M(3,9);
   A(41,56) = M(3,10);
   A(42,55) = M(3,10);
   A(43,54) = M(3,10);
   A(44,53) = M(3,10);
   A(45,52) = M(3,10);
   A(46,51) = M(3,10);
   A(47,50) = M(3,10);
   A(48,49) = M(3,10);
   A(49,48) = M(3,10);
   A(50,47) = M(3,10);
   A(51,46) = M(3,10);
   A(52,45) = M(3,10);
   A(53,44) = M(3,10);
   A(54,43) = M(3,10);
   A(55,42) = M(3,10);
   A(56,41) = M(3,10);
   A(57,40) = M(3,10);
   A(58,39) = M(3,10);
   A(59,38) = M(3,10);
   A(60,37) = M(3,10);
   A(61,47) = M(4,1);
   A(62,39) = M(4,1);
   A(63,38) = M(4,1);
   A(64,37) = M(4,1);
   A(65,27) = M(4,1);
   A(66,26) = M(4,1);
   A(67,25) = M(4,1);
   A(68,24) = M(4,1);
   A(69,23) = M(4,1);
   A(70,22) = M(4,1);
   A(71,10) = M(4,1);
   A(72,9) = M(4,1);
   A(73,8) = M(4,1);
   A(74,7) = M(4,1);
   A(75,6) = M(4,1);
   A(76,5) = M(4,1);
   A(77,4) = M(4,1);
   A(78,3) = M(4,1);
   A(79,2) = M(4,1);
   A(80,1) = M(4,1);
   A(61,48) = M(4,2);
   A(62,41) = M(4,2);
   A(63,40) = M(4,2);
   A(64,38) = M(4,2);
   A(65,30) = M(4,2);
   A(66,29) = M(4,2);
   A(67,28) = M(4,2);
   A(68,26) = M(4,2);
   A(69,25) = M(4,2);
   A(70,23) = M(4,2);
   A(71,14) = M(4,2);
   A(72,13) = M(4,2);
   A(73,12) = M(4,2);
   A(74,11) = M(4,2);
   A(75,9) = M(4,2);
   A(76,8) = M(4,2);
   A(77,7) = M(4,2);
   A(78,5) = M(4,2);
   A(79,4) = M(4,2);
   A(80,2) = M(4,2);
   A(61,50) = M(4,3);
   A(62,44) = M(4,3);
   A(63,43) = M(4,3);
   A(64,40) = M(4,3);
   A(65,34) = M(4,3);
   A(66,33) = M(4,3);
   A(67,32) = M(4,3);
   A(68,29) = M(4,3);
   A(69,28) = M(4,3);
   A(70,25) = M(4,3);
   A(71,19) = M(4,3);
   A(72,18) = M(4,3);
   A(73,17) = M(4,3);
   A(74,16) = M(4,3);
   A(75,13) = M(4,3);
   A(76,12) = M(4,3);
   A(77,11) = M(4,3);
   A(78,8) = M(4,3);
   A(79,7) = M(4,3);
   A(80,4) = M(4,3);
   A(61,49) = M(4,4);
   A(62,42) = M(4,4);
   A(63,41) = M(4,4);
   A(64,39) = M(4,4);
   A(65,31) = M(4,4);
   A(66,30) = M(4,4);
   A(67,29) = M(4,4);
   A(68,27) = M(4,4);
   A(69,26) = M(4,4);
   A(70,24) = M(4,4);
   A(71,15) = M(4,4);
   A(72,14) = M(4,4);
   A(73,13) = M(4,4);
   A(74,12) = M(4,4);
   A(75,10) = M(4,4);
   A(76,9) = M(4,4);
   A(77,8) = M(4,4);
   A(78,6) = M(4,4);
   A(79,5) = M(4,4);
   A(80,3) = M(4,4);
   A(61,51) = M(4,5);
   A(62,45) = M(4,5);
   A(63,44) = M(4,5);
   A(64,41) = M(4,5);
   A(65,35) = M(4,5);
   A(66,34) = M(4,5);
   A(67,33) = M(4,5);
   A(68,30) = M(4,5);
   A(69,29) = M(4,5);
   A(70,26) = M(4,5);
   A(71,20) = M(4,5);
   A(72,19) = M(4,5);
   A(73,18) = M(4,5);
   A(74,17) = M(4,5);
   A(75,14) = M(4,5);
   A(76,13) = M(4,5);
   A(77,12) = M(4,5);
   A(78,9) = M(4,5);
   A(79,8) = M(4,5);
   A(80,5) = M(4,5);
   A(61,52) = M(4,6);
   A(62,46) = M(4,6);
   A(63,45) = M(4,6);
   A(64,42) = M(4,6);
   A(65,36) = M(4,6);
   A(66,35) = M(4,6);
   A(67,34) = M(4,6);
   A(68,31) = M(4,6);
   A(69,30) = M(4,6);
   A(70,27) = M(4,6);
   A(71,21) = M(4,6);
   A(72,20) = M(4,6);
   A(73,19) = M(4,6);
   A(74,18) = M(4,6);
   A(75,15) = M(4,6);
   A(76,14) = M(4,6);
   A(77,13) = M(4,6);
   A(78,10) = M(4,6);
   A(79,9) = M(4,6);
   A(80,6) = M(4,6);
   A(61,53) = M(4,7);
   A(62,49) = M(4,7);
   A(63,48) = M(4,7);
   A(64,47) = M(4,7);
   A(65,42) = M(4,7);
   A(66,41) = M(4,7);
   A(67,40) = M(4,7);
   A(68,39) = M(4,7);
   A(69,38) = M(4,7);
   A(70,37) = M(4,7);
   A(71,31) = M(4,7);
   A(72,30) = M(4,7);
   A(73,29) = M(4,7);
   A(74,28) = M(4,7);
   A(75,27) = M(4,7);
   A(76,26) = M(4,7);
   A(77,25) = M(4,7);
   A(78,24) = M(4,7);
   A(79,23) = M(4,7);
   A(80,22) = M(4,7);
   A(61,54) = M(4,8);
   A(62,51) = M(4,8);
   A(63,50) = M(4,8);
   A(64,48) = M(4,8);
   A(65,45) = M(4,8);
   A(66,44) = M(4,8);
   A(67,43) = M(4,8);
   A(68,41) = M(4,8);
   A(69,40) = M(4,8);
   A(70,38) = M(4,8);
   A(71,35) = M(4,8);
   A(72,34) = M(4,8);
   A(73,33) = M(4,8);
   A(74,32) = M(4,8);
   A(75,30) = M(4,8);
   A(76,29) = M(4,8);
   A(77,28) = M(4,8);
   A(78,26) = M(4,8);
   A(79,25) = M(4,8);
   A(80,23) = M(4,8);
   A(61,55) = M(4,9);
   A(62,52) = M(4,9);
   A(63,51) = M(4,9);
   A(64,49) = M(4,9);
   A(65,46) = M(4,9);
   A(66,45) = M(4,9);
   A(67,44) = M(4,9);
   A(68,42) = M(4,9);
   A(69,41) = M(4,9);
   A(70,39) = M(4,9);
   A(71,36) = M(4,9);
   A(72,35) = M(4,9);
   A(73,34) = M(4,9);
   A(74,33) = M(4,9);
   A(75,31) = M(4,9);
   A(76,30) = M(4,9);
   A(77,29) = M(4,9);
   A(78,27) = M(4,9);
   A(79,26) = M(4,9);
   A(80,24) = M(4,9);
   A(61,56) = M(4,10);
   A(62,55) = M(4,10);
   A(63,54) = M(4,10);
   A(64,53) = M(4,10);
   A(65,52) = M(4,10);
   A(66,51) = M(4,10);
   A(67,50) = M(4,10);
   A(68,49) = M(4,10);
   A(69,48) = M(4,10);
   A(70,47) = M(4,10);
   A(71,46) = M(4,10);
   A(72,45) = M(4,10);
   A(73,44) = M(4,10);
   A(74,43) = M(4,10);
   A(75,42) = M(4,10);
   A(76,41) = M(4,10);
   A(77,40) = M(4,10);
   A(78,39) = M(4,10);
   A(79,38) = M(4,10);
   A(80,37) = M(4,10);

   % [ U,S,V] = svd(A);
   % S = diag(S)'; S(50:56)/S(1)
   % v = V(:,56);
   % condit = [S(55)/S(1), S(56)/S(55)];
   % floor(10*sqrt(sum(V(:,53:56)'.^2)))
   % floor(10*abs(V(:,53:56)'))
   % floor(-log(abs(V(:,53:56)')))

   % Find `null space basis' N by pseudoinverse. Actually it spans a
   % little more than the true null space, but all roots do still lie
   % in it.
   % N = [-A(:,1:52) \ A(:,53:56); eye(4) ];
   % N'
   N = [-pinv(A(:,1:52),norm(A,1)*1e-10) * A(:,53:56); eye(4) ];
   % norm(N*v(53:56)-v)
   % [v,N*v(53:56),N*v(53:56)-v]'
   
   condv = 1e100;
   for trial=1:4
      %p = randn(1,4);
      p = [randn(1,3),0];

      % Form nonzero part of multiplication matrix by p*mu for 
      % random p, and reduce modulo N to get 4x4 reduced multiplication
      % matrix E. Eigenvectors of this give scaled [mu1,mu2,mu3,1] of
      % potential roots.

      % --- u1^2  u1*u2 u1*u3 u2^2  u2*u3 u3^2  u1    u2    u3    1 ---
      Mp = [p(1), p(2), p(3),  0,    0,    0,  p(4),  0,    0,    0; ... %u1
	     0,   p(1),  0,   p(2), p(3),  0,   0,   p(4),  0,    0; ... %u2
	     0,    0,   p(1),  0,   p(2), p(3), 0,    0,   p(4),  0; ... %u3 
	     0,    0,    0,    0,    0,    0,  p(1), p(2), p(3), p(4) ]; %1

      E = Mp * N(47:56,:);
      % [E*u,u(1)/u(4)*u]
      % [V,S] = eig(E);
      [D1,E1] = balance(E);
      [V1,S] = eig(E1);
      V = D1*V1;
      V = V ./ (ones(4,1)*sqrt(sum(V.* conj(V))));
      cv = cond(V);
      % [norm(N,1),cv]
      % diag(S)'

      if (cv<condv) 
	 VV = V; 
	 SS = diag(S);
	 condv = cv;
	 if (condv<10) break; end;
      end;
   end;
   % [norm(N,1),condv]

   % Regenerate the 56-D vector of monomials [mu^5..mu^0]
   % corresponding to each real root using the null space basis N, 
   % and select the root that is closest to the monomial consistency
   % constraints as the solution.
   nt = 1e10*ones(4,1);
   nt2 = 1e10*ones(4,3);
   for i=1:4
      if (imag(SS(i))==0)
	 v = VV(:,i);
	 v2 = [v(1)*v(1:3); v(2)*v(2:3); v(3)^2];
	 t2 = v(4)*N(47:52,:)*v - v2;
	 v3 = [v(1)*v2; v(2)*v2(4:6); v(3)^3];
	 v4 = [v(1)*v3; v(2)*v3(7:10); v(3)^4];
	 v5 = [v(1)*v4; v(2)*v4(11:15); v(3)^5];
	 t5 = [v(4)^4*N(1:21,:); v(4)^3*N(22:36,:); ...
	       v(4)^2*N(37:46,:); v(4)*N(47:52,:)] ...
		* v - [v5;v4;v3;v2];
         r2 = M * [v2; v];
	 nt(i) = norm(t2,1); 
	 nt2(i,:) = [nt(i),norm(t5,1),norm(r2,1)];
	 %	 t5'
	 %	 if (nt1<nt) uu = v; nt = nt1; end;
      end;
   end;
   % nt2
   nt1 = min(nt);
   uu = [];
   for i=1:4
      if (nt(i)<1e2*nt1 & nt(i)<1e10) 
	 uu = [uu,VV(:,i)]; 
      end;
   end;
   %[u,uu],
   % min(norm(uu-u),norm(uu+u))
   u = uu;
   condit = [norm(N,1),condv,reshape(nt2,1,12)];
end;
