% 52x49 sparse multiresultant matrix for 4 point DLT-like pose method,
% found using Emeris' INCRES code. The Macaulay one is only a bit bigger
% at 80x56.

% Using the DLT on 4 known 3D points, the possible projection matrices
% have been reduced to a 4 parameter family P = u1*P1+..+u4*P4 where
% (u1..u4) are unknown parameters.  We use 4 supplementary calibration
% constraints aspect_ratio = 1, skew=0, principal_point=(0,0) (i.e.
% w(1,1)-w(2,2) = w(1,2) = w(1,3) = w(2,3) = 0 where w is the DIAC) to
% eliminate (u1..u4) and solve for P, camera pose, calibration, etc.
% The redundancy is 1 (since u1..u4 are only defined up to scale), so a
% linear solution is possible.  The constraints are all quadratic in
% (u1..u4). Each row of the 4x10 input matrix M represents one of these
% homogeneous quadratics (i.e. it is a flattened 4x4 symmetric
% matrix). The output multiresultant matrix has rank 79 in the noiseless
% case (but alas only 76 for coplanar points). The last 4 components of
% its null vector are (...u1..u4), which in turn gives P, pose, etc. The
% matrix was calculated with Maple using my extended multiresultant
% routine xmultires().

% Columns of A and entries of v correspond to these monomials:
% [u1^3*u2*u3, u1^2*u2^3, u1^2*u2^2*u3, u1^2*u2*u3^2, u1^2*u3^3, u1*u2^4, 
%  u1*u2^3*u3, u1*u2^2*u3^2, u1*u2*u3^3, u1*u3^4, u2^5, u2^4*u3, u2^3*u3^2,
%  u2^2*u3^3, u2*u3^4, u3^5, u1^3*u3*u4, u1^2*u2^2*u4, u1^2*u2*u3*u4, 
%  u1^2*u3^2*u4, u1*u2^3*u4, u1*u2^2*u3*u4, u1*u2*u3^2*u4, u1*u3^3*u4, 
%  u2^4*u4, u2^3*u3*u4, u2^2*u3^2*u4, u2*u3^3*u4, u3^4*u4, u1^3*u4^2, 
%  u1^2*u2*u4^2, u1^2*u3*u4^2, u1*u2^2*u4^2, u1*u2*u3*u4^2, 
%  u1*u3^2*u4^2,u2^3*u4^2, u2^2*u3*u4^2, u2*u3^2*u4^2, u3^3*u4^2, u1^2*u4^3, 
%  u1*u2*u4^3, u1*u3*u4^3, u2^2*u4^3, u2*u3*u4^3, u3^2*u4^3, u1*u4^4, 
%  u2*u4^4, u3*u4^4, u4^5]

function [u,cond] = dlt4multires49(M);
   A = zeros(52,49);
      A(1,1) = M(1,1);
      A(2,32) = M(1,1);
      A(3,19) = M(1,1);
      A(4,20) = M(1,1);
      A(8,4) = M(1,1);
      A(12,40) = M(1,1);
      A(16,31) = M(1,1);
      A(25,5) = M(1,1);
      A(33,18) = M(1,1);
      A(37,3) = M(1,1);
      A(41,2) = M(1,1);
      A(44,30) = M(1,1);
      A(48,17) = M(1,1);
      A(1,3) = M(1,2);
      A(2,34) = M(1,2);
      A(3,22) = M(1,2);
      A(4,23) = M(1,2);
      A(8,8) = M(1,2);
      A(12,41) = M(1,2);
      A(16,33) = M(1,2);
      A(25,9) = M(1,2);
      A(33,21) = M(1,2);
      A(37,7) = M(1,2);
      A(41,6) = M(1,2);
      A(44,31) = M(1,2);
      A(48,19) = M(1,2);
      A(1,7) = M(1,3);
      A(2,37) = M(1,3);
      A(3,26) = M(1,3);
      A(4,27) = M(1,3);
      A(8,13) = M(1,3);
      A(12,43) = M(1,3);
      A(16,36) = M(1,3);
      A(25,14) = M(1,3);
      A(33,25) = M(1,3);
      A(37,12) = M(1,3);
      A(41,11) = M(1,3);
      A(44,33) = M(1,3);
      A(48,22) = M(1,3);
      A(1,4) = M(1,4);
      A(2,35) = M(1,4);
      A(3,23) = M(1,4);
      A(4,24) = M(1,4);
      A(8,9) = M(1,4);
      A(12,42) = M(1,4);
      A(16,34) = M(1,4);
      A(25,10) = M(1,4);
      A(33,22) = M(1,4);
      A(37,8) = M(1,4);
      A(41,7) = M(1,4);
      A(44,32) = M(1,4);
      A(48,20) = M(1,4);
      A(1,8) = M(1,5);
      A(2,38) = M(1,5);
      A(3,27) = M(1,5);
      A(4,28) = M(1,5);
      A(8,14) = M(1,5);
      A(12,44) = M(1,5);
      A(16,37) = M(1,5);
      A(25,15) = M(1,5);
      A(33,26) = M(1,5);
      A(37,13) = M(1,5);
      A(41,12) = M(1,5);
      A(44,34) = M(1,5);
      A(48,23) = M(1,5);
      A(1,9) = M(1,6);
      A(2,39) = M(1,6);
      A(3,28) = M(1,6);
      A(4,29) = M(1,6);
      A(8,15) = M(1,6);
      A(12,45) = M(1,6);
      A(16,38) = M(1,6);
      A(25,16) = M(1,6);
      A(33,27) = M(1,6);
      A(37,14) = M(1,6);
      A(41,13) = M(1,6);
      A(44,35) = M(1,6);
      A(48,24) = M(1,6);
      A(1,19) = M(1,7);
      A(2,42) = M(1,7);
      A(3,34) = M(1,7);
      A(4,35) = M(1,7);
      A(8,23) = M(1,7);
      A(12,46) = M(1,7);
      A(16,41) = M(1,7);
      A(25,24) = M(1,7);
      A(33,33) = M(1,7);
      A(37,22) = M(1,7);
      A(41,21) = M(1,7);
      A(44,40) = M(1,7);
      A(48,32) = M(1,7);
      A(1,22) = M(1,8);
      A(2,44) = M(1,8);
      A(3,37) = M(1,8);
      A(4,38) = M(1,8);
      A(8,27) = M(1,8);
      A(12,47) = M(1,8);
      A(16,43) = M(1,8);
      A(25,28) = M(1,8);
      A(33,36) = M(1,8);
      A(37,26) = M(1,8);
      A(41,25) = M(1,8);
      A(44,41) = M(1,8);
      A(48,34) = M(1,8);
      A(1,23) = M(1,9);
      A(2,45) = M(1,9);
      A(3,38) = M(1,9);
      A(4,39) = M(1,9);
      A(8,28) = M(1,9);
      A(12,48) = M(1,9);
      A(16,44) = M(1,9);
      A(25,29) = M(1,9);
      A(33,37) = M(1,9);
      A(37,27) = M(1,9);
      A(41,26) = M(1,9);
      A(44,42) = M(1,9);
      A(48,35) = M(1,9);
      A(1,34) = M(1,10);
      A(2,48) = M(1,10);
      A(3,44) = M(1,10);
      A(4,45) = M(1,10);
      A(8,38) = M(1,10);
      A(12,49) = M(1,10);
      A(16,47) = M(1,10);
      A(25,39) = M(1,10);
      A(33,43) = M(1,10);
      A(37,37) = M(1,10);
      A(41,36) = M(1,10);
      A(44,46) = M(1,10);
      A(48,42) = M(1,10);
      A(5,1) = M(2,1);
      A(6,19) = M(2,1);
      A(9,4) = M(2,1);
      A(10,5) = M(2,1);
      A(11,40) = M(2,1);
      A(13,31) = M(2,1);
      A(14,32) = M(2,1);
      A(24,20) = M(2,1);
      A(34,18) = M(2,1);
      A(38,3) = M(2,1);
      A(42,2) = M(2,1);
      A(45,30) = M(2,1);
      A(49,17) = M(2,1);
      A(5,3) = M(2,2);
      A(6,22) = M(2,2);
      A(9,8) = M(2,2);
      A(10,9) = M(2,2);
      A(11,41) = M(2,2);
      A(13,33) = M(2,2);
      A(14,34) = M(2,2);
      A(24,23) = M(2,2);
      A(34,21) = M(2,2);
      A(38,7) = M(2,2);
      A(42,6) = M(2,2);
      A(45,31) = M(2,2);
      A(49,19) = M(2,2);
      A(5,7) = M(2,3);
      A(6,26) = M(2,3);
      A(9,13) = M(2,3);
      A(10,14) = M(2,3);
      A(11,43) = M(2,3);
      A(13,36) = M(2,3);
      A(14,37) = M(2,3);
      A(24,27) = M(2,3);
      A(34,25) = M(2,3);
      A(38,12) = M(2,3);
      A(42,11) = M(2,3);
      A(45,33) = M(2,3);
      A(49,22) = M(2,3);
      A(5,4) = M(2,4);
      A(6,23) = M(2,4);
      A(9,9) = M(2,4);
      A(10,10) = M(2,4);
      A(11,42) = M(2,4);
      A(13,34) = M(2,4);
      A(14,35) = M(2,4);
      A(24,24) = M(2,4);
      A(34,22) = M(2,4);
      A(38,8) = M(2,4);
      A(42,7) = M(2,4);
      A(45,32) = M(2,4);
      A(49,20) = M(2,4);
      A(5,8) = M(2,5);
      A(6,27) = M(2,5);
      A(9,14) = M(2,5);
      A(10,15) = M(2,5);
      A(11,44) = M(2,5);
      A(13,37) = M(2,5);
      A(14,38) = M(2,5);
      A(24,28) = M(2,5);
      A(34,26) = M(2,5);
      A(38,13) = M(2,5);
      A(42,12) = M(2,5);
      A(45,34) = M(2,5);
      A(49,23) = M(2,5);
      A(5,9) = M(2,6);
      A(6,28) = M(2,6);
      A(9,15) = M(2,6);
      A(10,16) = M(2,6);
      A(11,45) = M(2,6);
      A(13,38) = M(2,6);
      A(14,39) = M(2,6);
      A(24,29) = M(2,6);
      A(34,27) = M(2,6);
      A(38,14) = M(2,6);
      A(42,13) = M(2,6);
      A(45,35) = M(2,6);
      A(49,24) = M(2,6);
      A(5,19) = M(2,7);
      A(6,34) = M(2,7);
      A(9,23) = M(2,7);
      A(10,24) = M(2,7);
      A(11,46) = M(2,7);
      A(13,41) = M(2,7);
      A(14,42) = M(2,7);
      A(24,35) = M(2,7);
      A(34,33) = M(2,7);
      A(38,22) = M(2,7);
      A(42,21) = M(2,7);
      A(45,40) = M(2,7);
      A(49,32) = M(2,7);
      A(5,22) = M(2,8);
      A(6,37) = M(2,8);
      A(9,27) = M(2,8);
      A(10,28) = M(2,8);
      A(11,47) = M(2,8);
      A(13,43) = M(2,8);
      A(14,44) = M(2,8);
      A(24,38) = M(2,8);
      A(34,36) = M(2,8);
      A(38,26) = M(2,8);
      A(42,25) = M(2,8);
      A(45,41) = M(2,8);
      A(49,34) = M(2,8);
      A(5,23) = M(2,9);
      A(6,38) = M(2,9);
      A(9,28) = M(2,9);
      A(10,29) = M(2,9);
      A(11,48) = M(2,9);
      A(13,44) = M(2,9);
      A(14,45) = M(2,9);
      A(24,39) = M(2,9);
      A(34,37) = M(2,9);
      A(38,27) = M(2,9);
      A(42,26) = M(2,9);
      A(45,42) = M(2,9);
      A(49,35) = M(2,9);
      A(5,34) = M(2,10);
      A(6,44) = M(2,10);
      A(9,38) = M(2,10);
      A(10,39) = M(2,10);
      A(11,49) = M(2,10);
      A(13,47) = M(2,10);
      A(14,48) = M(2,10);
      A(24,45) = M(2,10);
      A(34,43) = M(2,10);
      A(38,37) = M(2,10);
      A(42,36) = M(2,10);
      A(45,46) = M(2,10);
      A(49,42) = M(2,10);
      A(7,1) = M(3,1);
      A(15,40) = M(3,1);
      A(18,32) = M(3,1);
      A(19,31) = M(3,1);
      A(20,20) = M(3,1);
      A(21,19) = M(3,1);
      A(23,4) = M(3,1);
      A(28,5) = M(3,1);
      A(35,18) = M(3,1);
      A(39,3) = M(3,1);
      A(43,2) = M(3,1);
      A(46,30) = M(3,1);
      A(51,17) = M(3,1);
      A(7,3) = M(3,2);
      A(15,41) = M(3,2);
      A(18,34) = M(3,2);
      A(19,33) = M(3,2);
      A(20,23) = M(3,2);
      A(21,22) = M(3,2);
      A(23,8) = M(3,2);
      A(28,9) = M(3,2);
      A(35,21) = M(3,2);
      A(39,7) = M(3,2);
      A(43,6) = M(3,2);
      A(46,31) = M(3,2);
      A(51,19) = M(3,2);
      A(7,7) = M(3,3);
      A(15,43) = M(3,3);
      A(18,37) = M(3,3);
      A(19,36) = M(3,3);
      A(20,27) = M(3,3);
      A(21,26) = M(3,3);
      A(23,13) = M(3,3);
      A(28,14) = M(3,3);
      A(35,25) = M(3,3);
      A(39,12) = M(3,3);
      A(43,11) = M(3,3);
      A(46,33) = M(3,3);
      A(51,22) = M(3,3);
      A(7,4) = M(3,4);
      A(15,42) = M(3,4);
      A(18,35) = M(3,4);
      A(19,34) = M(3,4);
      A(20,24) = M(3,4);
      A(21,23) = M(3,4);
      A(23,9) = M(3,4);
      A(28,10) = M(3,4);
      A(35,22) = M(3,4);
      A(39,8) = M(3,4);
      A(43,7) = M(3,4);
      A(46,32) = M(3,4);
      A(51,20) = M(3,4);
      A(7,8) = M(3,5);
      A(15,44) = M(3,5);
      A(18,38) = M(3,5);
      A(19,37) = M(3,5);
      A(20,28) = M(3,5);
      A(21,27) = M(3,5);
      A(23,14) = M(3,5);
      A(28,15) = M(3,5);
      A(35,26) = M(3,5);
      A(39,13) = M(3,5);
      A(43,12) = M(3,5);
      A(46,34) = M(3,5);
      A(51,23) = M(3,5);
      A(7,9) = M(3,6);
      A(15,45) = M(3,6);
      A(18,39) = M(3,6);
      A(19,38) = M(3,6);
      A(20,29) = M(3,6);
      A(21,28) = M(3,6);
      A(23,15) = M(3,6);
      A(28,16) = M(3,6);
      A(35,27) = M(3,6);
      A(39,14) = M(3,6);
      A(43,13) = M(3,6);
      A(46,35) = M(3,6);
      A(51,24) = M(3,6);
      A(7,19) = M(3,7);
      A(15,46) = M(3,7);
      A(18,42) = M(3,7);
      A(19,41) = M(3,7);
      A(20,35) = M(3,7);
      A(21,34) = M(3,7);
      A(23,23) = M(3,7);
      A(28,24) = M(3,7);
      A(35,33) = M(3,7);
      A(39,22) = M(3,7);
      A(43,21) = M(3,7);
      A(46,40) = M(3,7);
      A(51,32) = M(3,7);
      A(7,22) = M(3,8);
      A(15,47) = M(3,8);
      A(18,44) = M(3,8);
      A(19,43) = M(3,8);
      A(20,38) = M(3,8);
      A(21,37) = M(3,8);
      A(23,27) = M(3,8);
      A(28,28) = M(3,8);
      A(35,36) = M(3,8);
      A(39,26) = M(3,8);
      A(43,25) = M(3,8);
      A(46,41) = M(3,8);
      A(51,34) = M(3,8);
      A(7,23) = M(3,9);
      A(15,48) = M(3,9);
      A(18,45) = M(3,9);
      A(19,44) = M(3,9);
      A(20,39) = M(3,9);
      A(21,38) = M(3,9);
      A(23,28) = M(3,9);
      A(28,29) = M(3,9);
      A(35,37) = M(3,9);
      A(39,27) = M(3,9);
      A(43,26) = M(3,9);
      A(46,42) = M(3,9);
      A(51,35) = M(3,9);
      A(7,34) = M(3,10);
      A(15,49) = M(3,10);
      A(18,48) = M(3,10);
      A(19,47) = M(3,10);
      A(20,45) = M(3,10);
      A(21,44) = M(3,10);
      A(23,38) = M(3,10);
      A(28,39) = M(3,10);
      A(35,43) = M(3,10);
      A(39,37) = M(3,10);
      A(43,36) = M(3,10);
      A(46,46) = M(3,10);
      A(51,42) = M(3,10);
      A(17,40) = M(4,1);
      A(22,1) = M(4,1);
      A(26,32) = M(4,1);
      A(27,20) = M(4,1);
      A(29,31) = M(4,1);
      A(30,5) = M(4,1);
      A(31,19) = M(4,1);
      A(32,4) = M(4,1);
      A(36,18) = M(4,1);
      A(40,3) = M(4,1);
      A(47,30) = M(4,1);
      A(50,2) = M(4,1);
      A(52,17) = M(4,1);
      A(17,41) = M(4,2);
      A(22,3) = M(4,2);
      A(26,34) = M(4,2);
      A(27,23) = M(4,2);
      A(29,33) = M(4,2);
      A(30,9) = M(4,2);
      A(31,22) = M(4,2);
      A(32,8) = M(4,2);
      A(36,21) = M(4,2);
      A(40,7) = M(4,2);
      A(47,31) = M(4,2);
      A(50,6) = M(4,2);
      A(52,19) = M(4,2);
      A(17,43) = M(4,3);
      A(22,7) = M(4,3);
      A(26,37) = M(4,3);
      A(27,27) = M(4,3);
      A(29,36) = M(4,3);
      A(30,14) = M(4,3);
      A(31,26) = M(4,3);
      A(32,13) = M(4,3);
      A(36,25) = M(4,3);
      A(40,12) = M(4,3);
      A(47,33) = M(4,3);
      A(50,11) = M(4,3);
      A(52,22) = M(4,3);
      A(17,42) = M(4,4);
      A(22,4) = M(4,4);
      A(26,35) = M(4,4);
      A(27,24) = M(4,4);
      A(29,34) = M(4,4);
      A(30,10) = M(4,4);
      A(31,23) = M(4,4);
      A(32,9) = M(4,4);
      A(36,22) = M(4,4);
      A(40,8) = M(4,4);
      A(47,32) = M(4,4);
      A(50,7) = M(4,4);
      A(52,20) = M(4,4);
      A(17,44) = M(4,5);
      A(22,8) = M(4,5);
      A(26,38) = M(4,5);
      A(27,28) = M(4,5);
      A(29,37) = M(4,5);
      A(30,15) = M(4,5);
      A(31,27) = M(4,5);
      A(32,14) = M(4,5);
      A(36,26) = M(4,5);
      A(40,13) = M(4,5);
      A(47,34) = M(4,5);
      A(50,12) = M(4,5);
      A(52,23) = M(4,5);
      A(17,45) = M(4,6);
      A(22,9) = M(4,6);
      A(26,39) = M(4,6);
      A(27,29) = M(4,6);
      A(29,38) = M(4,6);
      A(30,16) = M(4,6);
      A(31,28) = M(4,6);
      A(32,15) = M(4,6);
      A(36,27) = M(4,6);
      A(40,14) = M(4,6);
      A(47,35) = M(4,6);
      A(50,13) = M(4,6);
      A(52,24) = M(4,6);
      A(17,46) = M(4,7);
      A(22,19) = M(4,7);
      A(26,42) = M(4,7);
      A(27,35) = M(4,7);
      A(29,41) = M(4,7);
      A(30,24) = M(4,7);
      A(31,34) = M(4,7);
      A(32,23) = M(4,7);
      A(36,33) = M(4,7);
      A(40,22) = M(4,7);
      A(47,40) = M(4,7);
      A(50,21) = M(4,7);
      A(52,32) = M(4,7);
      A(17,47) = M(4,8);
      A(22,22) = M(4,8);
      A(26,44) = M(4,8);
      A(27,38) = M(4,8);
      A(29,43) = M(4,8);
      A(30,28) = M(4,8);
      A(31,37) = M(4,8);
      A(32,27) = M(4,8);
      A(36,36) = M(4,8);
      A(40,26) = M(4,8);
      A(47,41) = M(4,8);
      A(50,25) = M(4,8);
      A(52,34) = M(4,8);
      A(17,48) = M(4,9);
      A(22,23) = M(4,9);
      A(26,45) = M(4,9);
      A(27,39) = M(4,9);
      A(29,44) = M(4,9);
      A(30,29) = M(4,9);
      A(31,38) = M(4,9);
      A(32,28) = M(4,9);
      A(36,37) = M(4,9);
      A(40,27) = M(4,9);
      A(47,42) = M(4,9);
      A(50,26) = M(4,9);
      A(52,35) = M(4,9);
      A(17,49) = M(4,10);
      A(22,34) = M(4,10);
      A(26,48) = M(4,10);
      A(27,45) = M(4,10);
      A(29,47) = M(4,10);
      A(30,39) = M(4,10);
      A(31,44) = M(4,10);
      A(32,38) = M(4,10);
      A(36,43) = M(4,10);
      A(40,37) = M(4,10);
      A(47,46) = M(4,10);
      A(50,36) = M(4,10);
      A(52,42) = M(4,10);

   [U,S,V] = svd(A);
   S = diag(S)';
   cond = [S(48)/S(1), S(48)/S(49)];
   v = V(:,49);
   % v = v .* D';

   % To get most accurate ratios of u.i's, select largest 
   % column from uu = const * [u1;u2;u3;u4] * [u1^2*u2*u3,u2^4,u3^4,u4^4]

   uu = [v([ 1,  3,  4, 19]), ...
         v([ 6, 11, 12, 25]), ...
	 v([10, 15, 16, 29]), ...
	 v([46, 47, 48, 49])];

   nu = sum(uu.^2);
   % uu ./ (ones(4,1)*sqrt(nu))
   i = 1;
   for j=2:4 
      if (nu(j)>nu(i)) i=j; end;
   end;
   u = uu(:,i);
end;
